<?php

namespace Drupal\in;

use Drupal\Core\Asset\AssetDumperInterface;
use Drupal\Core\File\FileSystemInterface;
use Drupal\Core\Messenger\MessengerInterface;
use Drupal\Core\State\StateInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;

/**
 * Handle rendering for different pieces of the Important Notification module.
 */
final class InRenderer {

  use StringTranslationTrait;

  /**
   * Construct a new InRenderer object.
   */
  public function __construct(
    private StateInterface $state,
    protected MessengerInterface $messenger,
    protected AssetDumperInterface $dumper,
    protected readonly FileSystemInterface $fileSystem
  ) {}

  /**
   * Build out the important notifications container.
   *
   * @param array $vars
   *   A renderable array representing the top of the page.
   *
   * @see in_page_top()
   * @see hook_page_top()
   */
  public function buildNotifications(array &$vars): void {
    $configs = $this->state->get('in.state.configs', []);
    $notifications = [];
    foreach ($configs as $id => $config) {
      $markup = $this->getMarkup($config);
      if ($markup) {
        $notifications[$id] = [
          '#type' => 'markup',
          '#markup' => $markup,
        ];
      }
    }

    if ($notifications) {
      $vars['important_notification'] = [
        '#type' => 'container',
        '#attributes' => ['id' => 'i-notification-wrapper'],
        'notification' => $notifications,
        '#attached' => [
          'library' => [
            'in/in.container',
          ],
        ],
        // @todo '#access' => \Drupal::currentUser()->hasPermission('access toolbar'), ?
        //      '#cache' => [
        //        'keys' => ['in'],
        //        'contexts' => ['user.permissions'],
        //      ],
      ];
    }
  }

  /**
   * Returns template for the important notification.
   *
   * @param array $config
   *
   * @return string
   */
  private function getMarkup(array $config): string {
    if (!$config['enabled']) {
      return '';
    }
    $type = $config['type'] ?? 'default';
    $res = '<div class="i-notification in-type-' . $type . '">';
    $res .= $config['text']['value'] ?? '';
    $res .= '</div>';
    return $res;
  }

  /**
   * Generates new CSS file.
   *
   * @param array $configs
   *
   * @return void
   */
  public function generateCss(array $configs) {
    try {
      $data = '';
      foreach ($configs as $config) {
        $data .= sprintf('.in-type-%s { background-color: %s; color: %s; }',
          $config['machine_name'],
          $config['color_back'],
          $config['color_text']
        );
      }

      // Delete old file.
      $old_css = $this->state->get(InHelper::IN_STATE_CSS, '');
      if ($old_css) {
        $this->fileSystem->delete($old_css);
        $this->fileSystem->delete($old_css . '.gz');
      }

      // Dump the CSS file.
      $uri = $this->dumper->dump($data, 'css');
      if ($uri) {
        // Save new file.
        $this->state->set(InHelper::IN_STATE_CSS, $uri);
      }
      else {
        $this->messenger->addError($this->t('Something went wrong while generating the CSS file.'));
      }
    }
    catch (\Exception $e) {
      $this->messenger->addError($e->getMessage());
    }
  }

}
